SystemOrganization addCategory: #Flow!
SystemOrganization addCategory: #'Flow-API'!
SystemOrganization addCategory: #'Flow-Core'!
SystemOrganization addCategory: #'Flow-Errors'!
SystemOrganization addCategory: #'Flow-Handlers'!
SystemOrganization addCategory: #'Flow-Models'!
SystemOrganization addCategory: #'Flow-Tutorial'!

Exception subclass: #FlowError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Errors'!

FlowError subclass: #FlowModelDataError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Errors'!

FlowError subclass: #FlowModelGetterIsMissing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Errors'!

FlowError subclass: #FlowModelMetadataError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Errors'!

ZnWebSocketDelegate subclass: #FlowWebSocketDelegate
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowWebSocketDelegate methodsFor: 'actions' stamp: 'sebastiansastre 10/20/2013 11:57'!
addHandler: aHandler

	^ self handlers add: aHandler! !

!FlowWebSocketDelegate methodsFor: 'actions' stamp: 'sebastiansastre 10/20/2013 13:29'!
continuation
	"Return a block that defines what happens after a successful connection upgrade.
	We will receive a ready-to-use instanciated WebSocket.
	We pass on our log support and delegate processing to our handler."

	^ [ :webSocket |
			webSocket log: self log.
			self onClient: webSocket ]! !

!FlowWebSocketDelegate methodsFor: 'actions' stamp: 'sebastiansastre 10/20/2013 12:00'!
conventionalRequest: aRequest

	| reaction |

	reaction := self reactionFor: aRequest.

	reaction ifNil:[^ZnResponse notFound: aRequest uri].	

	^ reaction 
		value: aRequest 
		value: (ZnResponse new
				statusLine: (ZnStatusLine ok);
				headers: ZnHeaders defaultResponseHeaders;
				yourself)
! !

!FlowWebSocketDelegate methodsFor: 'actions' stamp: 'sebastiansastre 10/20/2013 12:01'!
handleRequest: request
	"Server delegate entry point"
	
	(prefix isNil or: [ prefix = request uri pathSegments ])
		ifFalse: [ ^ ZnResponse notFound: request uri ].
	^ (self isValidWebSocketRequest: request)
		ifTrue: [ self webSocketResponseForRequest: request ]
		ifFalse: [ self conventionalRequest: request ]! !

!FlowWebSocketDelegate methodsFor: 'accessing' stamp: 'sebastiansastre 10/20/2013 11:58'!
handlers 

	^ handlers ifNil:[self initializeHandlers]! !

!FlowWebSocketDelegate methodsFor: 'initialization' stamp: 'sebastiansastre 10/20/2013 11:58'!
initializeHandlers

	^ handlers := OrderedCollection new! !

!FlowWebSocketDelegate methodsFor: 'actions' stamp: 'sebastiansastre 10/20/2013 13:32'!
onClient: aWebSocket

	self websocketsHandler onClient: aWebSocket ! !

!FlowWebSocketDelegate methodsFor: 'actions' stamp: 'sebastiansastre 10/21/2013 06:04'!
onStop

	self handlers do:[:e| e onStop ]
	! !

!FlowWebSocketDelegate methodsFor: 'accessing' stamp: 'sebastiansastre 10/20/2013 11:58'!
reactionFor: aRequest
	"Answer a reaction for aRequest (or nil if can't find one)"
	
	^ self handlers
			detect:[:each| each isHandlerFor: aRequest]
			ifNone:[nil]

! !

!FlowWebSocketDelegate methodsFor: 'actions' stamp: 'sebastiansastre 10/21/2013 06:07'!
stop

	self onStop! !

!FlowWebSocketDelegate methodsFor: 'actions' stamp: 'sebastiansastre 10/20/2013 13:57'!
websocketsHandler
	
	^ self handlers 
		detect: [ :e| e handlesWebSockets ]
		ifNone: [ self error: 'There is no websocket handler in this server?' ]! !

!VirtualMachine class methodsFor: '*flow' stamp: 'sebastiansastre 8/3/2012 11:39'!
maxExternalSemaphores: aSize 
	"This method should never be called as result of normal program
	execution. If it is however, handle it differently:
	- In development, signal an error to promt user to set a bigger size
	at startup immediately.
	- In production, accept the cost of potentially unhandled interrupts,
	but log the action for later review.
	
	See comment in maxExternalObjectsSilently: why this behaviour is
	desirable, "
	"Can't find a place where development/production is decided.
	Suggest Smalltalk image inProduction, but use an overridable temp
	meanwhile. "
	| inProduction |
	self maxExternalSemaphores
		ifNil: [^ 0].
	inProduction := true.
	^ inProduction
		ifTrue: [self maxExternalSemaphoresSilently: aSize.
			self crTrace: 'WARNING: Had to increase size of semaphore signal handling table due to many external objects concurrently in use';
				 crTrace: 'You should increase this size at startup using #maxExternalObjectsSilently:';
				 crTrace: 'Current table size: ' , self maxExternalSemaphores printString]
		ifFalse: ["Smalltalk image"
			self error: 'Not enough space for external objects, set a larger size at startup!!'
			"Smalltalk image"]! !

Object subclass: #Flow
	instanceVariableNames: ''
	classVariableNames: 'Packages Repository'
	poolDictionaries: ''
	category: 'Flow-Core'!

!Flow commentStamp: 'sebsatiansastre 3/7/2012 17:38' prior: 0!
AmberMP this software adds to Amber a Model-Presenter pattern.
This class is only a development support so it can be used with Kaliningrad.

!

!Flow class methodsFor: 'testing' stamp: 'sebastiansastre 1/11/2013 09:13'!
isProduction

	^ ConfigurationFiles at: 'Flow' at: 'isProduction'! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 10/20/2013 11:54'!
makeDispatcher
	"Answers the dispatcher that has mapped the flow API."
		
	^ FlowWebSocketDelegate new ! !

!Flow class methodsFor: 'accessing' stamp: 'sebastiansastre 10/18/2013 12:38'!
port

	^ ConfigurationFiles at: 'Flow' at: 'port' ! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 10/18/2013 12:37'!
reset

	ConfigurationFiles reset! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 4/8/2012 19:09'!
restart

	self stop.
	self startHttpd! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 3/10/2012 21:04'!
start

	self restart! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 11/11/2013 01:02'!
startHttpd

	(ZnServer startDefaultOn: self port)
		delegate: self makeDispatcher;
		register;
		yourself! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 1/13/2013 12:34'!
stop

	ConfigurationFiles reset.
	self stopHttpd ! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 10/27/2013 00:30'!
stopHttpd

	
	ZnServer managedServers do:[:e| e stop].
	
	ZnServer default ifNotNil:[
		ZnServer default delegate stop].
	
	ZnServer stopDefault. 
	
! !

Object subclass: #FlowAbstractAPI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-API'!

!FlowAbstractAPI class methodsFor: 'actions' stamp: 'sebastiansastre 4/11/2012 11:55'!
answerJSONFor: anObject
	"Produce the http response with anObject as JSON."

	^ ZnResponse ok: (ZnStringEntity text: anObject asJsonString)! !

!FlowAbstractAPI class methodsFor: 'actions' stamp: 'sebastiansastre 11/4/2013 22:22'!
reify: aModelClass from: aRequest 
	"Answers the reified model by using the (expected) JSON in aRequest"
	
	^ aModelClass fromJSON: aRequest contents keys first ! !

FlowAbstractAPI subclass: #FlowAbstractAPICommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-API'!

!FlowAbstractAPICommand class methodsFor: 'accessing' stamp: 'sebastiansastre 4/11/2012 14:22'!
actionFromMethod: aRequestMethod

	^ (self methodSelectors detect:[:e| e key = aRequestMethod]) value! !

!FlowAbstractAPICommand class methodsFor: 'accessing' stamp: 'sebastiansastre 6/28/2012 15:17'!
commandFor: aRequest 

	^ FlowAbstractAPICommand allSubclasses 
		detect:[:command| command isCommandFor: aRequest]
		ifNone:[nil] 
! !

!FlowAbstractAPICommand class methodsFor: 'accessing' stamp: 'sebastiansastre 5/17/2012 19:31'!
create: aModelClass on: aRequest 

	| newOne |

	newOne := aModelClass new.
	self onAfterCreate: newOne.
	
	^ self answerJSONFor: newOne 
	! !

!FlowAbstractAPICommand class methodsFor: 'testing' stamp: 'sebastiansastre 6/28/2012 15:16'!
isCommandFor: aRequest 

	^ false

! !

!FlowAbstractAPICommand class methodsFor: 'accessing' stamp: 'sebastiansastre 6/17/2012 15:32'!
methodSelectors

	^ {
	#GET -> #read.
	#POST -> #create.
	#PUT -> #save.	
	#DELETE -> #delete.	
	}! !

!FlowAbstractAPICommand class methodsFor: 'reactions' stamp: 'sebastiansastre 5/16/2012 21:05'!
onAfterCreate: aNewModel

	aNewModel save! !

FlowAbstractAPICommand subclass: #FlowRiakAPICommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-API'!

!FlowRiakAPICommand class methodsFor: 'actions' stamp: 'sebastiansastre 6/28/2012 15:01'!
handle: aRequest

	self subclassResponsibility ! !

FlowRiakAPICommand subclass: #FlowRiakModelAPI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-API'!

!FlowRiakModelAPI class methodsFor: 'actions' stamp: 'sebastiansastre 7/26/2012 13:15'!
handle: aRequest

	| modelClass sel |

	modelClass := self modelFor: aRequest.

	aRequest uri segments size > 3 
		ifTrue:[sel:= aRequest uri segments fourth]
		ifFalse:[sel := self actionFromMethod: aRequest method].

	^ modelClass 
		perform: (sel,'For:') asSymbol 
		with: aRequest! !

!FlowRiakModelAPI class methodsFor: 'testing' stamp: 'sebastiansastre 6/28/2012 15:00'!
isCommandFor: aRequest 

	^ (self modelFor: aRequest) notNil


! !

!FlowRiakModelAPI class methodsFor: 'accessing' stamp: 'sebastiansastre 6/28/2012 15:18'!
modelFor: aRequest

	^ nil! !

FlowAbstractAPI subclass: #FlowApplicationAPI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-API'!

FlowApplicationAPI subclass: #FlowRiakApplicationAPI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-API'!

!FlowRiakApplicationAPI class methodsFor: 'accessing' stamp: 'sebastiansastre 6/28/2012 14:18'!
commandsAPIClass

	^ self subclassResponsibility ! !

!FlowRiakApplicationAPI class methodsFor: 'accessing' stamp: 'sebastiansastre 6/28/2012 14:17'!
databaseFor: aRequest

	^ Flow riak sessionAt: #default on: 'flowing'
! !

!FlowRiakApplicationAPI class methodsFor: 'accessing' stamp: 'sebastiansastre 6/28/2012 14:17'!
databaseName

	^ 'flowing'! !

!FlowRiakApplicationAPI class methodsFor: 'actions' stamp: 'sebastiansastre 6/28/2012 19:17'!
handle: aRequest 

	| command |

	command := self commandsAPIClass commandFor: aRequest.

	command ifNil:[^ ZnResponse badRequest: aRequest].
	
	^ (self databaseFor: aRequest) do:[
		command handle: aRequest]! !

Object subclass: #FlowAbstractHandler
	instanceVariableNames: ''
	classVariableNames: 'MIMETypes'
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowAbstractHandler commentStamp: 'sebastiansastre 3/9/2012 07:22' prior: 0!
AmberMPRestfulHandler lays the foundation to maintain ProxyModel and PersistentModel objects between client and server!

FlowAbstractHandler subclass: #FlowAPIHandler
	instanceVariableNames: 'handlers applications'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowAPIHandler commentStamp: 'sebastiansastre 4/11/2012 11:34' prior: 0!
This handler is a hub that routes requests to the concrete handlers of the (relevant) application!

!FlowAPIHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 11:30'!
basePath 

	^ '/api'! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/10/2012 16:32'!
addApplication: anApplication

	^ self applications 
		at: anApplication appName 
		put: anApplication! !

!FlowAPIHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/10/2012 16:31'!
applicationFrom: aRequest
	"Answers the application to be used for the operations related to aRequest"
	
	^ self applications at: aRequest uri segments first ! !

!FlowAPIHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/10/2012 16:30'!
applications 

	^ applications ifNil:[self initializeApplications]! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/10/2012 16:45'!
create: aModelClass on: aRequest
	
	| newOne app |

	app := self applicationFrom: aRequest.
	newOne := aModelClass new.

	self onBeforeCreate: newOne on: app.

	^ self answerJSONFor: newOne! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/10/2012 11:47'!
delete: anId
	<delete>
	<path: '/{anId}'>
		
	| aModel |
	aModel := self readModel: anId.

	self onBeforeDelete: aModel.
	
	^ self answerJSONFor: aModel! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/11/2012 14:19'!
handle: aRequest with: aResponse
	
	| application action modelClass |

	(self isValid: aRequest) ifFalse:[
		^ ZnResponse badRequest: aRequest].
	
	application := self applicationFrom: aRequest.
	application ifNil:[
		^ ZnResponse badRequest: aRequest].

	^ application api handle: aRequest.
	
! !

!FlowAPIHandler methodsFor: 'initialization' stamp: 'sebastiansastre 4/10/2012 16:30'!
initializeApplications

	^ applications := Dictionary new! !

!FlowAPIHandler methodsFor: 'testing' stamp: 'sebastiansastre 1/8/2013 13:39'!
isAboutModel: aRequest
	"Answers true if aRequest it's about any of the models."

	^ (self modelClassFrom: aRequest) notNil! !

!FlowAPIHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 22:51'!
isHandlerFor: aRequest
	"Answers true if this handler should take care of aRequest.
	Note: subclasses should specialize"

	^ aRequest uri = self baseUri or:[
	(aRequest uri segments notNil and:[
	aRequest uri segments includes: self basePath allButFirst])]! !

!FlowAPIHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/11/2012 14:18'!
isValid: aRequest 

	^ (#(#GET #POST #DELETE #PUT) includes: aRequest method) and:[
	self isAboutModel: aRequest]! !

!FlowAPIHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/10/2012 11:47'!
modelClass

	^ self class modelClass! !

!FlowAPIHandler methodsFor: 'accessing' stamp: 'sebastiansastre 6/6/2013 16:41'!
modelClassFrom: aRequest 
	"Answers the model referred by aRequest (or nil if absent or invalid)"
	
	^ [self modelClasses 
		detect: [:e| (e name copyFrom: 3 to: e name size) = aRequest uri segments third]
		ifNone:[nil]] on: Error do:[:x| nil ]! !

!FlowAPIHandler methodsFor: 'accessing' stamp: 'sebastiansastre 11/10/2013 15:01'!
modelClasses

	^ FlowModel allSubclasses,
	MaplessMongo allSubclasses  ! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 4/10/2012 16:46'!
onBeforeCreate: newOne on: anApplication
	"This model has been created (and is about to be stored).
	This is a great chance to do something you could need about this."
	
	anApplication onBeforeCreate: newOne ! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 4/10/2012 16:23'!
onBeforeDelete: aModel on: aDatabase
	"This model is about to be deleted.
	This is a great chance to do something you could need about this."
	
	aDatabase delete: aModel! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 4/10/2012 16:24'!
onBeforeRead: anId on: aDatabase
	"This model is about to be read.
	This is a great chance to do something you could need about this."

	^ (aDatabase read: self modelClass atId: anId) onRead! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 4/10/2012 16:25'!
onBeforeUpdate: aModel on: aDatabase
	"This model is about to be updated.
	This is a great chance to do something you could need about this."
	
	aDatabase update: aModel! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 4/10/2012 16:26'!
onBeforeUpdateFrom: someJson on: aDatabase
	"A model is about to be updated.
	This is a great chance to do something you could need about this."

	| aModel |

	aModel := self reify: someJson.
	
	^ aDatabase update: aModel! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/10/2012 11:47'!
read: anId
	<get>
	<path: '/{anId}'>
	
	^ self answerJSONFor: (self readModel: anId)! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/10/2012 11:48'!
readModel: anId
	"Answers the persisted instance of the model found at anId (or nil)"

	^ self onBeforeRead: anId! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 11/4/2013 22:22'!
reify: someJSON
	"Answers a model reified from someJSON.
	AKA inflates that JSON :)"

	^ self modelClass fromJSON: someJSON! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/10/2012 11:48'!
update: anId
	<put>
	<path: '/{anId}'>
	
	| aModel json |
	json := 	self requestContext request rawBody.

	aModel := self onBeforeUpdateFrom: json.
	 	
	^ self answerJSONFor: aModel! !

!FlowAbstractHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 12:52'!
basePath

	^ self subclassResponsibility ! !

!FlowAbstractHandler class methodsFor: 'initialization' stamp: 'sebastiansastre 4/8/2012 21:09'!
initializeMimeTypes

	^ MIMETypes := self makeMIMETypes! !

!FlowAbstractHandler class methodsFor: 'actions' stamp: 'sebastiansastre 4/8/2012 23:03'!
makeMIMETypes 

	^ Dictionary new 
		at: 'txt' put: ZnMimeType textPlain;
		at: 'text' put: ZnMimeType textPlain;
		at: 'js' put: ZnMimeType textJavascript;
		at: 'html' put: ZnMimeType textHtml;
		at: 'css' put: ZnMimeType textCss;
		at: 'png' put: ZnMimeType imagePng;
		at: 'gif' put: ZnMimeType imageGif;
		at: 'jpg' put: ZnMimeType imageJpeg;
		at: 'jpeg' put: ZnMimeType imageJpeg;
		yourself 
		! !

!FlowAbstractHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/8/2012 21:09'!
mimeTypes

	^ MIMETypes ifNil:[self initializeMimeTypes]! !

!FlowAbstractHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 3/9/2012 18:09'!
modelClass 
	"Answers the class of the models manipulated by this handler.
	Suggested practice is: one handler per class model."
	
	self subclassResponsibility ! !

!FlowAbstractHandler class methodsFor: 'actions' stamp: 'sebastiansastre 4/8/2012 21:10'!
reset 

	MIMETypes := nil! !

!FlowAbstractHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 12:52'!
basePath

	^ self class basePath ! !

!FlowAbstractHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 12:05'!
baseUri

	^ self class basePath asZnUrl ! !

!FlowAbstractHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/8/2012 20:39'!
fileNotFound: aRequest

	ZnResponse notFound: aRequest uri
	! !

!FlowAbstractHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/8/2012 21:21'!
handle: aRequest with: aResponse

	^ ZnResponse ok: (ZnEntity html: '<h1>flow API</h1>')! !

!FlowAbstractHandler methodsFor: 'testing' stamp: 'sebastiansastre 10/20/2013 13:40'!
handlesWebSockets
	
	^ false! !

!FlowAbstractHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 19:22'!
hasPackageNameFor: aRequest

	^ false! !

!FlowAbstractHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 19:21'!
isCodeFor: aRequest

	^ false! !

!FlowAbstractHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 17:33'!
isFileHandlerFor: aRequest 
	"Answers true if the filename in aRequest is present among the files
	handled by this handler.
	Applicability: useful for writting a file you also serve with this handler.
	Flow uses this every time you commit code from the Amber's Browser."
	
	^ false! !

!FlowAbstractHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 11:50'!
isHandlerFor: aRequest
	"Answers true if this handler should take care of aRequest.
	Note: subclasses should specialize"
	
	^ false! !

!FlowAbstractHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/8/2012 23:03'!
mimetypeForFile: aFilename

	^ self class mimeTypes at: (aFilename basename copyAfterLast: $.)! !

!FlowAbstractHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 15:38'!
notFound: aRequest

	^ ZnResponse notFound: aRequest uri
	! !

!FlowAbstractHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/21/2013 06:06'!
onStop! !

!FlowAbstractHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/8/2012 23:10'!
respond: anEntity mimeType: aMIMEType

	^ ZnResponse ok: anEntity! !

!FlowAbstractHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/18/2013 14:22'!
value: aRequest value: aResponse
	"Handle aRequest with aResponse"

	^ self handle: aRequest with: aResponse
	! !

FlowAbstractHandler subclass: #FlowFilesHandler
	instanceVariableNames: 'base path files'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

FlowFilesHandler subclass: #FlowCodeHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowCodeHandler commentStamp: 'sebastiansastre 3/15/2012 16:31' prior: 0!
FlowCodeHandler 
		
This class handles your Amber's code.

Your code is treated differently depending if ti's about Amber itself or if it's about your project.

For Amber itself, this handler uses FlowLibrary and for your project it keeps a class var ProjectLibrary to manage it with your own library.
Note that it's always expected that you put your own library there.

Your own libraries for your own projects are the right place to configure the path where your Amber's application files are gowing to be read and write. Note that Flow manages the code for only one project at the time. If you have many projects in the same image you need to change it accordingly.

Example:

	FlowCodeHandler projectLibrary: MyGreatAmberProjectLibrary default


MyGreatAmberProjectLibrary class>>basePath
	
	^ '/Users/sebastian/Documents/git/myGreatAmberProject' asPath  
	
...and

FlowLibrary class>>basePath
	
	^ '/Users/sebastian/Documents/git/amber' asPath    <-- that's where you have your local git clone of your forked amber
!

!FlowCodeHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/18/2013 18:12'!
fileFrom: aRequest

	^ self path / aRequest uri segments last
	! !

!FlowCodeHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/18/2013 18:13'!
handle: aRequest with: aResponse

	| handler |
	
	(self isValid: aRequest) ifFalse:[
		^ ZnResponse badRequest: aRequest].

	handler := self handlerFor: aRequest.

	^ handler
		ifNil:[ZnResponse notFound: aRequest uri]
		ifNotNil:[handler writeContentFrom: aRequest]! !

!FlowCodeHandler methodsFor: 'accessing' stamp: 'sebastiansastre 10/18/2013 18:02'!
handlerFor: aRequest 
	"Answer which handler in this server can take care of aRequest
	(or nil if none found)."
	
	^ ZnServer default delegate handlers 
		detect:[:e| e isHandlerFor: aRequest]
		ifNone:[nil]
	! !

!FlowCodeHandler methodsFor: 'testing' stamp: 'sebastiansastre 10/18/2013 17:29'!
isHandlerFor: aRequest
	"Answers true if this handler should take care of aRequest.
	Note: subclasses should specialize"

	^ aRequest method = #PUT and:[
	(aRequest uri segments includes: 'lib')]! !

!FlowCodeHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 17:12'!
isValid: aRequest 

	^ aRequest method = #PUT! !

FlowCodeHandler subclass: #FlowJsCodeHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowJsCodeHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 11:31'!
basePath 

	^ '/code/js'! !

!FlowJsCodeHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/18/2013 18:02'!
handle: aRequest with: aResponse

	| handler |

	(self isValid: aRequest) ifFalse:[
		^ ZnResponse badRequest: aRequest].

	handler := self handlerFor: aRequest.

	^ handler
		ifNil:[ZnResponse notFound: aRequest uri]
		ifNotNil:[handler writeContentFrom: aRequest]! !

!FlowJsCodeHandler methodsFor: 'initialization' stamp: 'sebastiansastre 10/18/2013 18:08'!
initializePath

	^ path := FileLocator imageDirectory parent / 'service' / 'static' / 'lib' / 'amber' / 'js'! !

!FlowJsCodeHandler methodsFor: 'testing' stamp: 'sebastiansastre 8/5/2012 15:37'!
isHandlerFor: aRequest
	"Answers true if this handler should take care of aRequest.
	Note: subclasses should specialize"

	^ (super isHandlerFor: aRequest) and:[
	'*.js' match: aRequest uri segments last]! !

FlowCodeHandler subclass: #FlowStCodeHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowStCodeHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 11:31'!
basePath 

	^ '/code/st'! !

!FlowStCodeHandler methodsFor: 'initialization' stamp: 'sebastiansastre 10/18/2013 18:08'!
initializePath

	^ path := FileLocator imageDirectory parent / 'service' / 'static' / 'lib' / 'amber' / 'st'! !

!FlowStCodeHandler methodsFor: 'testing' stamp: 'sebastiansastre 8/5/2012 15:41'!
isHandlerFor: aRequest
	"Answers true if this handler should take care of aRequest.
	Note: subclasses should specialize"

	^ (super isHandlerFor: aRequest) and:[
	'*.st' match: aRequest uri segments last]! !

!FlowFilesHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 13:56'!
basePath 

	^ '/files'! !

!FlowFilesHandler class methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 09:54'!
for: aPath

	^ self new 
		path: aPath;
		yourself ! !

!FlowFilesHandler class methodsFor: 'actions' stamp: 'sebastiansastre 8/1/2012 16:49'!
for: aPath base: aString

	^ self new 
		base: aString;
		path: aPath;
		yourself ! !

!FlowFilesHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 19:23'!
packageName
	"Answers the name of the package that is managed by this handler (if any)"
	
	"Default is none"
	^ ''! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 8/1/2012 16:46'!
base

	^ base ifNil:[self initializeBase]! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 8/1/2012 16:48'!
base: aString

	base := aString 
	! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 8/1/2012 16:48'!
baseUri

	^ self base asZnUrl ! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 10:32'!
defaultFilename

	^ 'index.html'! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/8/2012 23:06'!
documentForFile: aFilename type: aMIMEType

	^ aFilename
		readStreamDo: [:stream | aMIMEType isBinary
				ifTrue: [stream contents]
				ifFalse: [stream contents asString]]! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 21:52'!
fileFrom: aRequest
	"Answers the local filename that is suposed to match the one at the URI in aRequest"
	self subclassResponsibility ! !

!FlowFilesHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/22/2013 21:40'!
fileNameFrom: aRequest
		
	| requestedPath absolute |

	absolute := self path asFileReference absolutePath.

	aRequest uri isDirectoryPath ifTrue:[
		^ absolute asFileReference / self defaultFilename].
	
	requestedPath := aRequest url segments.

	absolute asFileReference path segments
		do: [:part | (requestedPath includes: part)
				ifTrue: [requestedPath remove: part]].

	requestedPath
		do: [:part | absolute := absolute / part].

	^ absolute asFileReference! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 09:51'!
files 

	^ files ifNil:[self initializeFiles]! !

!FlowFilesHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/18/2013 17:30'!
getFiles 

	^ self path asFileReference exists 
		ifFalse:[#()]
		ifTrue:[self path asFileReference allChildren] 
	! !

!FlowFilesHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/18/2013 15:15'!
handle: aRequest with: aResponse

	| file fileStream mime entity response |

	(self isValid: aRequest) ifFalse:[
		^ ZnResponse badRequest: aRequest].

	file := self fileNameFrom: aRequest.

	file isDirectory ifTrue:[
		^ self 
			redirectFor: aRequest 
			with: (ZnResponse redirect: (self baseUri addPathSegment: #/)) ].

	file exists ifFalse: [
		^ self notFound: aRequest].
	
	mime := self mimetypeForFile: file.

	entity := ZnStreamingEntity type: mime length: file entry size.

	fileStream := file readStream binary.
	entity stream: fileStream.
	
	response := ZnResponse ok: entity.
	self onAboutToServe: file for: aRequest on: response.
	response headers at: 'Modification-Date' put: (ZnUtils httpDate:  file entry modification).
	

	^ response! !

!FlowFilesHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 19:24'!
hasPackageNameFor: aRequest

	^ self class packageName isEmpty
		ifTrue:[false]
		ifFalse:[(self class packageName,'*') match: aRequest uri segments last] ! !

!FlowFilesHandler methodsFor: 'initialization' stamp: 'sebastiansastre 8/1/2012 18:15'!
initializeBase

	^ base := self class basePath ! !

!FlowFilesHandler methodsFor: 'initialization' stamp: 'sebastiansastre 4/9/2012 09:51'!
initializeFiles 

	^ files := self getFiles! !

!FlowFilesHandler methodsFor: 'initialization' stamp: 'sebastiansastre 10/18/2013 18:07'!
initializePath

	^ path := (FileLocator imageDirectory / 'files') resolve asString! !

!FlowFilesHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 19:26'!
isCodeFor: aRequest

	^ ('*.js' match: aRequest uri segments last) or:[
	'*.st' match: aRequest uri segments last]! !

!FlowFilesHandler methodsFor: 'testing' stamp: 'sebastiansastre 1/8/2013 13:20'!
isFileHandlerFor: aRequest 
	"Answers true if the filename in aRequest is present among the files
	handled by this handler.
	Applicability: useful for writting a file you also serve with this handler.
	Flow uses this every time you commit code from the Amber's Browser."

	^ (self files anySatisfy:[:e| 
		e basename = aRequest uri segments last]) or:[
	((self isCodeFor: aRequest) and:[
	self hasPackageNameFor: aRequest])]! !

!FlowFilesHandler methodsFor: 'testing' stamp: 'sebastiansastre 10/18/2013 13:47'!
isHandlerFor: aRequest
	"Answers true if this handler should take care of aRequest.
	Note: subclasses should specialize"

	^ aRequest uri = self baseUri or:[
	(aRequest uri segments notNil and:[
	aRequest uri segments first = self base asZnUrl segments first])]! !

!FlowFilesHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 16:59'!
isValid: aRequest 

	^ aRequest method = #GET! !

!FlowFilesHandler methodsFor: 'reactions' stamp: 'sebastiansastre 1/9/2013 18:26'!
onAboutToServe: aFileName for: aRequest on: aResponse
	"If you want to send some header or cookie now is your last chance"! !

!FlowFilesHandler methodsFor: 'reactions' stamp: 'sebastiansastre 8/5/2012 02:22'!
onAboutToServe: aFileName on: aResponse
	"If you want to send some header or cookie now is your last chance"! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 09:50'!
path 

	^ path ifNil:[self initializePath]! !

!FlowFilesHandler methodsFor: 'accessing' stamp: 'sebastiansastre 8/1/2012 18:03'!
path: aPath 

	path := aPath.
	self reset! !

!FlowFilesHandler methodsFor: 'actions' stamp: 'sebastiansastre 8/2/2012 15:23'!
redirectFor: aRequest with: aResponse

	^ aResponse 
! !

!FlowFilesHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 09:53'!
reset

	files := nil! !

!FlowFilesHandler methodsFor: 'actions' stamp: 'sebastiansastre 10/18/2013 17:31'!
writeContentFrom: aRequest 

	| file content |

	file := (self files
			detect:[:each| each basename = aRequest uri segments last]
			ifNone:[self fileFrom: aRequest]) asFileReference. 

	file parent ensureDirectory. 
	
	aRequest contents isString
		ifTrue: [content := aRequest contents withInternetLineEndings]
		ifFalse: [content := aRequest contents].
		
	"For some reason if we don't delete it first, it gets write corrupted sometimes :/"
	file delete.
	file writeStreamDo:[:stream|
		stream << content].
	
	^ ZnResponse ok: (ZnEntity text: 'OK')
	
! !

FlowFilesHandler subclass: #FlowHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 17:29'!
basePath 

	^ '/amber'! !

!FlowHandler methodsFor: 'initialization' stamp: 'sebastiansastre 4/9/2012 17:30'!
initializePath
	"Answers the local path to flow.
	Note that what makes sense the most here is to point it to your local git clone."
	
	^ path := '/Users/sebastian/Documents/git/flow'
! !

FlowAbstractHandler subclass: #FlowRootHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowRootHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 11:38'!
basePath 

	^ '/'! !

!FlowRootHandler methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 14:11'!
handle: aRequest with: aResponse

	| entity |

	aRequest uri = '/favicon.ico' asZnUrl ifTrue:[
		entity := ZnEntity with: ZnConstants faviconBytes type: 'image/vnd.microsoft.icon'. 
		^ ZnResponse ok: entity].
	
	^ ZnResponse ok: (ZnEntity html: '<h1>flow</h1>')! !

!FlowRootHandler methodsFor: 'testing' stamp: 'sebastiansastre 4/9/2012 13:58'!
isHandlerFor: aRequest
	"Answers true if this handler should take care of aRequest.
	Note: subclasses should specialize"

	^ aRequest uri isSlash or:[
	aRequest uri = '/favicon.ico' asZnUrl]! !

Object subclass: #FlowDispatcherDelegate
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowDispatcherDelegate methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 11:36'!
addHandler: aHandler

	^ self handlers add: aHandler! !

!FlowDispatcherDelegate methodsFor: 'accessing' stamp: 'sebastiansastre 4/10/2012 16:33'!
api 

	^ self handlers detect:[:e| e isKindOf: FlowAPIHandler]! !

!FlowDispatcherDelegate methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 18:04'!
handleRequest: aRequest

	| reaction |

	reaction := self reactionFor: aRequest.

	reaction ifNil:[^ZnResponse notFound: aRequest uri].	

	^ reaction 
		value: aRequest 
		value: (ZnResponse new
				statusLine: (ZnStatusLine ok);
				headers: ZnHeaders defaultResponseHeaders;
				yourself)
! !

!FlowDispatcherDelegate methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 11:32'!
handlers 

	^ handlers ifNil:[self initializeHandlers]! !

!FlowDispatcherDelegate methodsFor: 'initialization' stamp: 'sebastiansastre 10/19/2013 16:31'!
initialize 
	"The dispachet is responsible for delegating who cares about what request.
	This basically configures the server."
	
	super initialize.
	
	self
		"addHandler: FlowAmberHandler new;
		addHandler: FlowAPIHandler new;"
		addHandler: FlowStCodeHandler new;
		addHandler: FlowJsCodeHandler new;
		addHandler: FlowRootHandler new;
		yourself ! !

!FlowDispatcherDelegate methodsFor: 'initialization' stamp: 'sebastiansastre 4/9/2012 11:36'!
initializeHandlers

	^ handlers := OrderedCollection new! !

!FlowDispatcherDelegate methodsFor: 'accessing' stamp: 'sebastiansastre 4/9/2012 14:10'!
reactionFor: aRequest
	"Answer a reaction for aRequest (or nil if can't find one)"
	
	^ self handlers
			detect:[:each| each isHandlerFor: aRequest]
			ifNone:[nil]

! !

!FlowDispatcherDelegate methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 11:36'!
removeHandler: aHandler

	^ self handlers remove: aHandler! !

Object subclass: #FlowModel
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Models'!

!FlowModel commentStamp: 'sebastiansastre 3/15/2012 21:49' prior: 0!
Flow models are meant to be treated as aggregates.

We don't care here about identity.

Why?

Because the idea is that you keep these objects small and uncomplicated enough to be more efficienty serialized/deserialized in JSON for using it is some sort of noSQL persistence support.!

FlowModel subclass: #FlowLogin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Tutorial'!

!FlowLogin methodsFor: 'initialization' stamp: 'sebastiansastre 4/9/2012 14:24'!
initialize

	super initialize.
	
	"We initialize this without user, meaning: signed out"
	self user: nil! !

!FlowLogin methodsFor: 'reactions' stamp: 'sebastiansastre 4/9/2012 14:24'!
onValidLogin: aLogin for: anUser

	self user: anUser.
	aLogin user: anUser.! !

!FlowLogin methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 14:24'!
printOn: aStream

	super printOn: aStream.
		
	aStream nextPutAll: '(',self email asString,')'! !

!FlowLogin methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 14:23'!
validateOn: aLogin
	"Validates this login with aLogin.
	If the validation is achieved, this login will have an user
	and stay ready to go. User will be nil otherwhise."

	| user |

	user := self database userAtEmail: self email.
	user ifNil:[
		self user: nil.
		^ self].

	self password = user password 
		ifTrue:[self onValidLogin: aLogin for: user]
		ifFalse:[self user: nil].
	
	 ! !

!FlowModel class methodsFor: 'actions' stamp: 'sebastiansastre 6/13/2012 21:18'!
answerJSONFor: anObject

	^ ZnResponse ok: (ZnStringEntity text: anObject asJsonString)
! !

!FlowModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 11:30'!
findAll 

	^ self database objectsAt: self! !

!FlowModel class methodsFor: 'accessing' stamp: 'sebastiansastre 5/17/2012 16:28'!
findAt: anId

	^ ((self database objectsAt: self ifAbsent:[^nil])
		detect: [:e| e id = anId] 
		ifNone: [nil]) onRead ! !

!FlowModel class methodsFor: 'accessing' stamp: 'sebastiansastre 11/4/2013 22:25'!
fromJSONObject: aJsonObject

	^ self basicNew 
		data: aJsonObject;
		onReified  
! !

!FlowModel class methodsFor: 'accessing' stamp: 'sebastiansastre 11/4/2013 22:25'!
fromJson: someJson

	^ self fromJSONObject: (OrderedJson readFrom: someJson readStream)! !

!FlowModel class methodsFor: 'accessing' stamp: 'sebastiansastre 11/4/2013 22:25'!
fromReified: aJsonObject 

	^ self fromJSONObject: aJsonObject ! !

!FlowModel class methodsFor: 'actions' stamp: 'sebastiansastre 3/4/2012 19:31'!
makeId

	^ UUID new asString36! !

!FlowModel methodsFor: 'testing' stamp: 'sebastiansastre 5/15/2012 20:10'!
= aModel 
	"Answers true if this model is the same as aModel.
	Note: as you can see, this won't care about identity (yes, that's intentional)."
	^ aModel class = self class and:[
	self id = aModel id]! !

!FlowModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 11:58'!
asJsonString
	
	self onAboutToJSON.
	
	^ super asJsonString! !

!FlowModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 10:43'!
at: aKey

	^ data at: aKey! !

!FlowModel methodsFor: 'actions' stamp: 'sebastiansastre 3/24/2012 10:43'!
at: aKey put: anObject

	^ data at: aKey put: anObject! !

!FlowModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 11:43'!
createdOn 

	| selector expects object|

	selector := #createdOn.
	expects := DateAndTime.

	object := self at: selector asString.
	object ifNil:[^nil].

	(object isKindOf: expects) ifTrue:[^object].

	^ self at: selector asString put: (self dateAndTimeAt: selector).
! !

!FlowModel methodsFor: 'accessing' stamp: 'sebastiansastre 7/26/2012 13:28'!
data

	^ data ifNil:[self initializeData]! !

!FlowModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 23:32'!
data: aJsonObject

	data := aJsonObject 
	! !

!FlowModel methodsFor: 'actions' stamp: 'sebastiansastre 3/24/2012 11:47'!
dateAndTimeAt: aSelector

	^ DateAndTime fromString: (self at: aSelector)! !

!FlowModel methodsFor: 'actions' stamp: 'sebastiansastre 11/4/2013 22:25'!
doesNotUnderstand: aMessage
	"The idea behind this DNU is to use the selector as setters or getter 
	delegating to data (aJsonObject)"
	
	| key part subModel |

	key := aMessage selector.
	key isUnary ifTrue: [
		part := data at: key asString.
		part ifNil:[^nil].
		
		(part class inheritsFrom: FlowRiakModel) ifTrue:[^ part].
			(part isKindOf: OrderedJsonObject)
				ifFalse: [^part]
				ifTrue: [
					part isEmpty ifTrue:[^nil].
					subModel := part at: 'modelClass' ifAbsent:[self error: 'Unexpected JSON'].
						^ self 
							at: key asString 
							put: ((Smalltalk at: subModel asSymbol) fromJSONObject: part)]].

	^ (key isKeyword and: [(key occurrencesOf: $:) = 1])
		ifTrue: [key := key allButLast asSymbol.
				data at: key asString put: aMessage arguments first]
		ifFalse: [super doesNotUnderstand: aMessage]
				
! !

!FlowModel methodsFor: 'actions' stamp: 'sebastiansastre 5/17/2012 16:28'!
fresh

	^ self class findAt: self id ! !

!FlowModel methodsFor: 'initialization' stamp: 'sebastiansastre 7/26/2012 16:54'!
initialize

	super initialize.
	
	"This will initialize data but only if strictly needed."
	self data.
		
	"We include the class name so we have the minimal metadata at consuming time"
	self modelClass ifNil:[self modelClass: self class name].

	self id ifNil:[self id: self class makeId].
	self createdOn ifNil:[self createdOn: DateAndTime now].
	
! !

!FlowModel methodsFor: 'initialization' stamp: 'sebastiansastre 7/26/2012 14:16'!
initializeData

	^ data := OrderedJsonObject new! !

!FlowModel methodsFor: 'initialization' stamp: 'sebastiansastre 7/26/2012 16:55'!
initializeOn: aJsonObject 
	"Makes this model to be what aJsonObject dictates it to be.
	Unwraps data if necessary."
	self halt.
	(aJsonObject at: '@data') isNil 
		ifTrue:[data := aJsonObject]
		ifFalse:[data := aJsonObject at: '@data']
	 ! !

!FlowModel methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 23:53'!
jsonWriteOn: aStream

	data jsonWriteOn: aStream! !

!FlowModel methodsFor: 'accessing' stamp: 'sebastiansastre 4/24/2012 13:28'!
name 

	^ self at: 'name'! !

!FlowModel methodsFor: 'accessing' stamp: 'sebastiansastre 4/24/2012 13:28'!
name: aString 

	^ self at: 'name' put: aString! !

!FlowModel methodsFor: 'reactions' stamp: 'sebastiansastre 5/17/2012 17:00'!
onAboutToJSON
	"This model is about to be stringified in JSON.
	All inst var values that are expected to be model objects, need to be instanciated as expected."

	data keys do:[:key| |value|
		value := data at: key.
		"If the value is a js object proxy, we assume it's the data for a (composed) flow model, 
		so we call the getter in this receiver expecting to get it properly instantiated (because that's our convention here)."
		(value isKindOf: OrderedJsonObject) ifTrue:[
			(self respondsTo: key asSymbol) ifFalse:[FlowModelGetterIsMissing signal: 'Cannot find getter for: ', key].
			value := self perform: key asSymbol.
			value onAboutToJSON.
			self at: key put: value]].
! !

!FlowModel methodsFor: 'reactions' stamp: 'sebastiansastre 5/17/2012 16:37'!
onRead
	"This model has been readed.
	All inst var values that are expected to be model objects, need to be re-instanciated."

	"data keys do:[:key| |value|
		value := data at: key.
		value isPersistentModel ifTrue:[
			value := value fresh.
			self at: key put: value]]."! !

!FlowModel methodsFor: 'reactions' stamp: 'sebastiansastre 7/28/2012 13:12'!
onReified
	"Process the receiver just after receiving the reified object"

	! !

!FlowModel methodsFor: 'accessing' stamp: 'sebastiansastre 7/26/2012 16:55'!
subModelAt: aSelector
	"Answers the reified submodel (instantiating if necessary)."
	
	| subModelData json class |

	subModelData := data at: aSelector.
	subModelData ifNil:[^nil].

	subModelData isEmpty ifTrue:[^nil].
	
	class := subModelData at: 'modelClass' ifAbsent:[
		^ FlowModelDataError signal: 'Cannot find ', aSelector,'''s data'].

	class := Smalltalk at: class asSymbol ifAbsent:[
		FlowModelMetadataError signal: 'Cannot find ',aSelector,'''s metadata'].
	
	^ class fromReified: subModelData	! !

FlowModel subclass: #FlowPerson
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Tutorial'!

FlowModel subclass: #FlowToDo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Tutorial'!

FlowModel subclass: #FlowToDoList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Tutorial'!

FlowModel subclass: #FlowUser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Tutorial'!

!FlowUser commentStamp: 'sebastiansastre 4/9/2012 14:29' prior: 0!
This is an example of how you can implement Users in Flow.

This user has essentially an email and a password and a person.

Take a look at the person accessor to see an example on how it's getting reified lazily on demand.

When models have submodels, you don't need to map anything, just use this kind of methods where you customize #selector and #expects and Flow will take care of their JSON reification/serialization.
!

!FlowUser methodsFor: 'actions' stamp: 'sebastiansastre 4/9/2012 14:25'!
printOn: aStream

	super printOn: aStream.
		
	aStream nextPutAll: '(',self email asString,')'! !

Object subclass: #FlowSimpleDb
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Core'!

FlowSimpleDb class
	instanceVariableNames: 'instance'!

!FlowSimpleDb commentStamp: 'sebastiansastre 3/15/2012 21:25' prior: 0!
This is a simple singleton holding the PersistentModel's you send to them.

Please note that, by now, we only care about persisten models at a level of the "valuesof things"

Don't expect identity to be the rule to compare things here but equal values.

Example: 

	guy := PersonMock new id: '1234'; yourself.
	dude := PersonMock new id: '1234'; yourself.
	
	guy == dude "this is false"
	guy = dude "this is true"
	
	db add: dude.   
	db exists: dude.  "this is true"
	db exists: guy.  "this is also true"

	db remove: guy.
	db exists: dude.  "this is false now"
	db exists: guy.  "this is false now"
			
!

FlowSimpleDb class
	instanceVariableNames: 'instance'!

!FlowSimpleDb class methodsFor: 'initialization' stamp: 'sebastiansastre 3/4/2012 19:15'!
initializeInstance

	^ instance := self new! !

!FlowSimpleDb class methodsFor: 'accessing' stamp: 'sebsatiansastre 3/7/2012 15:09'!
instance

	^ instance ifNil:[self initializeInstance]! !

!FlowSimpleDb class methodsFor: 'actions' stamp: 'sebastiansastre 3/4/2012 19:15'!
reset

	instance := nil! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/17/2012 16:25'!
add: aPersistentModel
	
	| collection |
	
	collection := self objects 
					at: aPersistentModel class 
					ifAbsentPut:[OrderedCollection new].

	(collection includes: aPersistentModel) ifFalse:[
		self addSubModelsFor: aPersistentModel.
		collection add: aPersistentModel].
	
	^ aPersistentModel! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 4/25/2012 15:34'!
addSubModelsFor: aPersistentModel
	"If any of the values in aPersistentModel is also aFlowPersistentModel,
	then add it to the database."

	aPersistentModel data values do:[:each|
		(each isKindOf: FlowModel) ifTrue:[
			self add: each]]
	
	! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/17/2012 19:28'!
at: anId of: aPersistentModelClass

	^ (self objectsAt: aPersistentModelClass ifAbsent:[^nil])
		detect: [:e| e id = anId] 
		ifNone: [nil]! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:43'!
create: anObject
	
	^ self save: anObject! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/23/2012 20:34'!
delete: aPersistentModel

	^ self remove: aPersistentModel! !

!FlowSimpleDb methodsFor: 'testing' stamp: 'sebastiansastre 3/15/2012 21:19'!
exists: aPersistentModel

	| instances |
	instances := self objects 
					at: aPersistentModel class 
					ifAbsent:[nil].

	^ instances isNil 
		ifTrue: [false] 
		ifFalse: [instances includes: aPersistentModel]! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 4/12/2012 08:55'!
find: aModelClass from: start to: end

	| instances |

	instances := self findAll: aModelClass.

	^instances copyFrom: start to: (end min: instances size)! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 4/12/2012 11:40'!
find: aModelClass with: property value: anObject

	^ (self findAll: aModelClass)
		detect: [:e| (e perform: property) = anObject ] 
		ifNone: [nil]! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 4/12/2012 08:56'!
findAll: aModelClass 

	^ self objectsAt: aModelClass! !

!FlowSimpleDb methodsFor: 'initialization' stamp: 'sebastiansastre 3/9/2012 18:15'!
initializeObjects

	^ objects := Dictionary new! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/9/2012 18:21'!
objects 

	^ objects ifNil:[self initializeObjects]! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 4/12/2012 08:57'!
objectsAt: aClass

	^ self objects at: aClass ifAbsent:[#()]! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/17/2012 19:27'!
objectsAt: aClass ifAbsent: aBlock

	^ self objects at: aClass ifAbsent: aBlock! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 4/10/2012 16:39'!
read: aModelClass atId: anId
		
	^ self at: anId of: aModelClass! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:43'!
readAt: anId of: aPersistentModelClass
 
	^ self at: anId of: aPersistentModelClass ! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:19'!
remove: aPersistentModel
	
	| collection |
	
	collection := self objects 
					at: aPersistentModel class 
					ifAbsentPut:[OrderedCollection new].

	^ collection remove: aPersistentModel ifAbsent:[nil]! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/15/2012 21:57'!
replace: aPersistentModel
	"Replaces the stored instance that has 
	the same id than aPersistentModel with aPersistentModel."

	| instances old |

	instances := self objectsAt: aPersistentModel class.
	old := instances detect: [:e| e id = aPersistentModel id].

	instances replaceAll: old with: aPersistentModel.
	
	^ aPersistentModel 
	! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:19'!
save: anObject
	
	^ self update: anObject! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:47'!
update: aPersistentModel

	^ (self exists: aPersistentModel)
		ifTrue: [self replace: aPersistentModel]
		ifFalse: [self add: aPersistentModel]! !

!String methodsFor: '*flow' stamp: 'sas 10/27/2013 11:00'!
toUtf8
	"Answers the string which is the result of decoding the receiver in utf8"
	
	^ (GRCodec forEncoding: 'utf-8') encode: self! !
